<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;


use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;

class Repository extends \Doctrine\ORM\EntityRepository {

    const STATUS_LOGICAL_DELETED = LogicalDeleted::STATUS_LOGICAL_DELETED;

    public function getRepositoryData($repositoryId) {
        $query = <<<SQL
        SELECT REPOSITORY.NAME AS NAME,
        REPOSITORY.URL AS URL
        FROM REPOSITORY
        WHERE REPOSITORY.ID = :REPOSITORY_ID
        AND REPOSITORY.STATUS <> :LOGICAL_DELETE
SQL;
        $params = array(
            'REPOSITORY_ID' => $repositoryId,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function findRepositoriesByResource($resource, $project) {
        $query = <<<SQL
            SELECT      R.NAME AS REPOSITORY_NAME, R.ID AS REPOSITORY_ID, R.URL AS REPOSITORY_URL, DIR.ID,
                        DIR.LOCAL_DIRECTORY, DIR.LINUX_DIRECTORY
            FROM        REPOSITORY R
            JOIN        DIRECTORY DIR           ON (DIR.REPOSITORY = R.ID)
            WHERE       DIR.RESOURCE_T = :RESOURCE_ID AND R.PROJECT = :PROJECT_ID
            AND R.STATUS <> :LOGICAL_DELETE
SQL;

        $params = array(
            'RESOURCE_ID' => $resource,
            'PROJECT_ID' => $project,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED
        );
        return $connection = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    public function getRepositoriesByTaskId($taskId,$resourceId) {
        $query = <<<SQL
            SELECT      R.NAME AS REPOSITORY_NAME, R.ID AS REPOSITORY_ID, R.URL AS REPOSITORY_URL
            FROM        REPOSITORY R
            JOIN        TASK TAS  ON (TAS.PROJECT = R.PROJECT)
            JOIN        DIRECTORY DIR ON (DIR.REPOSITORY = R.ID AND DIR.RESOURCE_T = :RESOURCE_ID)
            WHERE       TAS.ID = :TASK_ID
            AND         R.STATUS <> :LOGICAL_DELETE
            GROUP BY    R.NAME , R.ID, R.URL
SQL;

        $params = array(
            'TASK_ID' => $taskId,
            'RESOURCE_ID' => $resourceId,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED
        );
        return $connection = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findAllRepositories($organization, $filterString = ''){

        $query = <<<SQL
            SELECT MAX(R.NAME) AS NAME, MIN(R.ID) AS ID, R.URL, MAX(R.DESCRIPTION) AS DESCRIPTION
            FROM REPOSITORY R
            WHERE R.ORGANIZATION = :ORGANIZATION
             AND R.STATUS <> :LOGICAL_DELETE
             $filterString
         GROUP BY R.URL
SQL;

        $params = array(
            'ORGANIZATION' => $organization,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function getProjectRepositories($projectId,$filter) {
        $query = <<<SQL
            SELECT  R.NAME AS NAME, R.ID AS ID, R.DESCRIPTION AS DESCRIPTION, R.URL AS URL, R.STATUS AS STATUS
            FROM        REPOSITORY R
            INNER JOIN  PROJECT P ON P.ID = R.PROJECT
            WHERE P.ID = :PROJECT_ID AND R.STATUS <> :LOGICAL_DELETE
            $filter
SQL;
        $params = array(
            'PROJECT_ID' => $projectId,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED
        );
        return $connection = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findRepositoriesWithFilter($filter) {
        $query = <<<SQL
            SELECT R.NAME, R.ID, R.DESCRIPTION,
                   R.URL, R.STATUS, R.PROJECT
              FROM REPOSITORY R
             WHERE 1 = 1
               AND R.STATUS <> 'D'
                   $filter
SQL;
        return $connection = $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function findSubReleasesAndRepositoriesByRepositoryIds($repositoryIds, $productVersionIdDev = null, $originProductVersionId = null) {
        $versionDevFilter = "";
        if($productVersionIdDev) {
            $versionDevFilter = "AND S.PRODUCT_VERSION_ID = $productVersionIdDev";
        } else if($originProductVersionId) {
            $versionDevFilter = "AND S.PRODUCT_VERSION_ID IS NULL";
        }
        $query = <<<SQL
			SELECT SUBRELEASE_ID, RTRIM(XMLAGG(XMLELEMENT(e, REPOSITORY_ID, ',').EXTRACT('//text()') ORDER BY REPOSITORY_ID).GetClobVal(),',') AS REPOSITORY_IDS
              FROM (SELECT R.ID AS REPOSITORY_ID, S.ID AS SUBRELEASE_ID
                      FROM REPOSITORY R
                     INNER JOIN PROJECT P ON P.ID = R.PROJECT
                     INNER JOIN SUBRELEASE S ON S.PROJECT_ID = P.ID
                     WHERE R.ID IN (:REPOSITORY_IDS)
                       AND S.IS_ACTIVE = 'Y'
                       AND S.STATUS IN ('T', 'P', 'C', 'E')
                           $versionDevFilter)
             GROUP BY SUBRELEASE_ID
SQL;
        $params = array(
            'REPOSITORY_IDS' => $repositoryIds
        );
        $paramTypes = array(
            'REPOSITORY_IDS' => Connection::PARAM_INT_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    /**
     * @param $productIds
     * @param $deployIds
     * @return array
     */
    public function getMigrationRepositoriesInfoWithModules($productIds, $deployIds) {
        $query = "SELECT RPT.NAME, RPT.URL, PRP.ID,
                         PRP.DSDIRRELMIGBD AS MIG_RELATIVE_PATH,
                         DRP.INITIAL_MIGRATION_NUMBER,
                         REGEXP_REPLACE(PATH, (SELECT REP.PATH
                                                 FROM REPOSITORY REP
                                                WHERE REP.PRODUCT = RPT.PRODUCT
                                                  AND REP.STATUS = 'A'
                                                  AND REP.IS_MASTER = 'Y'), '', 1, 1) AS PATH
                    FROM OM_PRODUTO PDT
              INNER JOIN OM_PRODUTO_REPOSITORIO PRP ON PRP.NRPRODUTO_ID = PDT.ID
              INNER JOIN REPOSITORY RPT ON RPT.ID = PRP.NREXTERNAL_REPOSITORY_ID
               LEFT JOIN DEPLOY_REPOSITORY DRP ON DRP.EXTERNAL_REPOSITORY_ID = PRP.ID
                                              AND DRP.DEPLOY_ID IN (:DEPLOY_IDS)
                   WHERE NREXTERNAL_PRODUCT_ID IN (:PRODUCT_IDS)
                     AND RPT.STATUS = 'A'
                     AND PRP.IDATIVO = 'S'";

        $params = array(
            'PRODUCT_IDS' => $productIds,
            'DEPLOY_IDS'  => $deployIds
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getDeployMigration($repositoryName, $initNumber, $pendFiles)
    {
        $query = <<<SQL
		    SELECT MTD.ID, MTD.SCRIPT, MTD.DSDESESTRUTURA, MTD.IDRESULTSCRIPT, 
                   MTD.NRSCRIPT, MTD.REPOSITORIO_ID, MTR.DSURLREPOSITORIO,
                   CASE WHEN MTD.NRSCRIPT IN ($pendFiles)
                        THEN 'S'
                        ELSE 'N'
                    END AS IDARQPENDENTE
              FROM MT_DESIGN_ESTRUTURA MTD
        INNER JOIN MT_REPOSITORIO MTR ON MTR.ID = MTD.REPOSITORIO_ID
             WHERE MTD.IDATIVO = 'S'
               AND MTD.IDRESULTSCRIPT = 'ES'
               AND MTR.DSURLREPOSITORIO = :REPOSITORY_NAME
               AND (MTD.NRSCRIPT > TO_NUMBER(:INIT_NUMBER) OR MTD.NRSCRIPT IN ($pendFiles))
          ORDER BY MTD.NRSCRIPT
SQL;
        $param = array(
            'REPOSITORY_NAME' => $repositoryName,
            'INIT_NUMBER'     => $initNumber
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }
}